﻿Imports System
Imports System.Threading
Imports System.Runtime.InteropServices
Imports System.Text
Imports System.IO

Public Class PingTray

    Dim OptionExePath As String 'Ping.exeのパス
    Dim OptionLogDir As String 'ログフォルダのパス
    Dim OptionTarget As String 'Ping送信先IPアドレス
    Dim OptionTimeout As Integer 'タイムアウトミリ秒数
    Dim OptionInterval As Integer 'Ping送信タイマーのインターバル秒数
    Dim OptionLogDay As Integer 'ログファイル保有日数
    Dim OptionLogInterval As Integer 'ログファイル削除タイマーのインターバル秒数

    'INIファイル読み込み関連
    Private Declare Ansi Function GetPrivateProfileString Lib "KERNEL32.DLL" _
        Alias "GetPrivateProfileStringA" ( _
            <MarshalAs(UnmanagedType.LPStr)> ByVal lpApplicationName As String, _
            <MarshalAs(UnmanagedType.LPStr)> ByVal lpKeyName As String, _
            <MarshalAs(UnmanagedType.LPStr)> ByVal lpDefault As String, _
            <MarshalAs(UnmanagedType.LPStr)> ByVal lpReturnedString As StringBuilder, ByVal nSize As Integer, _
            <MarshalAs(UnmanagedType.LPStr)> ByVal lpFileName As String _
    ) As Integer

    'INIファイル読み込み関連
    Private Function GetIniFileValue(ByVal IniAppName As String, ByVal IniKeyName As String, ByVal IniDefValue As String) As String

        Dim IniValue As StringBuilder = New StringBuilder(256)
        Dim i As Long

        i = GetPrivateProfileString(IniAppName,
                                    IniKeyName,
                                    IniDefValue,
                                    IniValue,
                                    200,
                                    Directory.GetCurrentDirectory & "\PingTray.ini"
                                    )

        GetIniFileValue = Trim$(IniValue.ToString())

    End Function

    'PingTrayを開始
    Private Sub PingTray_Load(ByVal sender As Object, ByVal e As EventArgs) Handles Me.Load
        '設定を読み込む
        Call ExecReadIni()

        ''ウインドウのタイトルをターゲット名にする
        Me.Text = OptionTarget

        ''タスクトレイアイコンのツールチップをターゲット名にする
        NotifyIcon1.Text = OptionTarget

        ''一回目のPingを実行する
        Call ExecPing()

        ''一回目のログ削除を実行する
        Call ExecDeleteLog()

        ''タイマーを開始する
        Timer1.Enabled = True
        Timer2.Enabled = True
    End Sub

    'タスクトレイアイコンをダブルクリックしたときの動作
    Private Sub NotifyIcon1_MouseDoubleClick(ByVal sender As System.Object, ByVal e As System.Windows.Forms.MouseEventArgs) Handles NotifyIcon1.MouseDoubleClick
        'ウインドウが最小化状態の時
        If Me.WindowState = FormWindowState.Minimized Then
            'ウインドウを通常化する
            Call MainWindowNormalize()
        Else
            'ウインドウを最小化する
            Call MainWindowMinimize()
        End If
    End Sub

    '最小化ボタンをクリックしたときの動作
    Private Sub Button1_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles Button1.Click
        '最小化処理を実行する
        Call MainWindowMinimize()
    End Sub

    '終了ボタン
    Private Sub Button2_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles Button2.Click
        '終了メッセージを表示する
        If MsgBox("PingTrayを終了しますか？", vbOKCancel + vbDefaultButton2, "PingTray") = vbOK Then
            Me.Close()
        End If
    End Sub

    'ウインドウの通常化処理
    Private Sub MainWindowNormalize()
        With Me
            'ウインドウを通常状態にする
            .WindowState = FormWindowState.Normal

            '画面の右下に配置する
            .Left = Screen.PrimaryScreen.WorkingArea.Width - .Width
            .Top = Screen.PrimaryScreen.WorkingArea.Height - .Height

            'タスクバーに表示するようにする
            .ShowInTaskbar = True

            'ウインドウを表示する
            .Visible = True

            'ウインドウにフォーカス
            .Activate()
        End With
    End Sub

    'ウインドウの最小化処理
    Private Sub MainWindowMinimize()
        With Me
            'タスクバーに表示しないようにする
            .ShowInTaskbar = False

            'ウインドウを非表示にする
            .Visible = False

            'ウインドウを最小化する
            .WindowState = FormWindowState.Minimized
        End With
    End Sub

    Private Sub ExecPing()
        'ログフォルダの作成
        Directory.CreateDirectory(OptionLogDir & "PingTray_" & Now.ToString("yyyyMMdd"))

        'Pingオブジェクトの作成
        Dim PingObject As New Net.NetworkInformation.Ping()

        'Pingを送信する
        Dim PingReply As Net.NetworkInformation.PingReply = PingObject.Send(OptionTarget, OptionTimeout)

        '結果をログに書き出す
        If PingReply.Status <> Net.NetworkInformation.IPStatus.Success Then
            'ログファイルを開く
            Dim LogSW As New StreamWriter(OptionLogDir & "PingTray_" &
                                                    Now.ToString("yyyyMMdd") &
                                                    "\" &
                                                    Now.ToString("yyyyMMdd") &
                                                    ".csv",
                                                    True,
                                                    Encoding.GetEncoding("shift_jis"))

            'ログ出力する
            LogSW.WriteLine(Chr(34) & Now.ToString("yyyy/MM/dd") & Chr(34) & "," &
                            Chr(34) & Now.ToString("HH:mm:ss") & Chr(34) & "," &
                            Chr(34) & OptionTarget & Chr(34) & "," &
                            Chr(34) & PingReply.Status.ToString & Chr(34) & "," &
                            Chr(34) & PingDescription(PingReply.Status.ToString) & Chr(34))

            'ログファイルを閉じる
            LogSW.Close()
        End If

        'Pingオブジェクトの開放
        PingObject.Dispose()
        PingObject = Nothing

        'ガベージコレクタ起動
        GC.Collect()
    End Sub

    'メインループ
    Private Sub Timer1_Tick(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles Timer1.Tick
        '一旦タイマーを止める
        Timer1.Enabled = False

        Debug.Print("Timer1 Tick")

        'Pingを実行する
        ExecPing()

        'タイマーを再開する
        Timer1.Enabled = True
    End Sub

    'サブループ
    Private Sub Timer2_Tick(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles Timer2.Tick
        '一旦タイマーを止める
        Timer2.Enabled = False

        Debug.Print("Timer2 Tick")

        'ログ削除を実行する
        ExecDeleteLog()

        'タイマーを再開する
        Timer2.Enabled = True
    End Sub

    'Pingのエラー詳細を出力する
    Private Function PingDescription(ByVal PingCode As String) As String
        Select Case PingCode
            Case "BadDestination", "11018"
                PingDescription = "宛先が正しくありません"
            Case "BadHeader", "11042"
                PingDescription = "Bad Header"
            Case "BadOption", "11007"
                PingDescription = "オプションが正しくありません"
            Case "BadRoute", "11012"
                PingDescription = "無効なルートです"
            Case "DestinationHostUnreachable", "11003"
                PingDescription = "宛先ホストに到達できません"
            Case "DestinationNetworkUnreachable", "11002"
                PingDescription = "宛先ネットワークに到達できません"
            Case "DestinationPortUnreachable", "11005"
                PingDescription = "宛先ポートに到達できません"
            Case "DestinationProhibited", "11004"
                PingDescription = "宛先プロトコルに到達できません"
            Case "DestinationProtocolUnreachable"
                PingDescription = "Destination Protocol Unreachable"
            Case "DestinationScopeMismatch"
                PingDescription = "Destination Scope Mismatch"
            Case "DestinationUnreachable"
                PingDescription = "Destination Unreachable"
            Case "HardwareError", "11008"
                PingDescription = "ハードウェア エラーです"
            Case "IcmpError"
                PingDescription = "ICMP Error"
            Case "NoResources", "11006"
                PingDescription = "リソースがありません"
            Case "PacketTooBig", "11009"
                PingDescription = "パケットが大きすぎます"
            Case "ParameterProblem", "11015"
                PingDescription = "パラメータに問題があります"
            Case "SourceQuench", "11016"
                PingDescription = "発信元が抑制されています"
            Case "TimedOut", "11010"
                PingDescription = "要求がタイムアウトしました"
            Case "TimeExceeded"
                PingDescription = "Time Exceeded"
            Case "TtlExpired", "11013"
                PingDescription = "転送中に TimeToLive 有効期限が切れました"
            Case "TtlReassemblyTimeExceeded", "11014"
                PingDescription = "再構築中に TimeToLive 有効期限が切れました"
            Case "Unknown"
                PingDescription = "Unknown"
            Case "UnrecognizedNextHeader"
                PingDescription = "Unrecognized Next Header"
            Case "11001"
                PingDescription = "バッファが小さすぎます"
            Case "11011"
                PingDescription = "要求が正しくありません"
            Case "11017"
                PingDescription = "オプションが大きすぎます"
            Case "11032"
                PingDescription = "IPSEC のネゴシエート中です"
            Case "11050"
                PingDescription = "一般エラーです"
            Case Else
                PingDescription = "エラー:" & PingCode.ToString
        End Select
    End Function

    Private Sub ExecReadIni()
        '設定ファイルの読み込み処理
        OptionLogDir = GetIniFileValue("PingTray", "LOGDIR", "C:\TEMP\")
        OptionTarget = GetIniFileValue("PingTray", "TARGET", "www.google.co.jp")
        OptionTimeout = CInt(GetIniFileValue("PingTray", "TIMEOUT", "4000"))
        OptionInterval = CInt(GetIniFileValue("PingTray", "INTERVAL", "1")) * 1000
        OptionLogDay = CInt(GetIniFileValue("PingTray", "LOGDAY", "30"))
        OptionLogInterval = CInt(GetIniFileValue("PingTray", "LOGINT", "360")) * 1000

        '指定したフォルダがない場合エラーで終了
        If Directory.Exists(GetIniFileValue("PingTray", "LOGDIR", "C:\TEMP\")) = False Then
            NotifyIcon1.Visible = False
            MsgBox("設定されているログフォルダが存在しません。設定(LOGDIR)を確認してください。",
                   vbOKOnly,
                   "PingTray")
            End
        End If

        '設定したインターバル(秒)をタイマーに設定する
        If Not OptionInterval > 0 Then
            NotifyIcon1.Visible = False

            MsgBox("インターバルの設定値が異常です。設定(INTERVAL)を確認してください。",
                   vbOKOnly,
                   "PingTray")
            End
        Else
            Timer1.Interval = OptionInterval
        End If

        'タイムアウト秒数をチェックする
        If Not OptionTimeout > 0 Then
            NotifyIcon1.Visible = False

            MsgBox("タイムアウトの設定値が異常です。設定(TIMEOUT)を確認してください。",
                   vbOKOnly,
                   "PingTray")
            End
        End If

        'ログファイル保有日数をチェックする
        If Not OptionLogDay > 0 Then
            NotifyIcon1.Visible = False

            MsgBox("ログファイル保管日数の設定値が異常です。設定(LOGDAY)を確認してください。",
                   vbOKOnly,
                   "PingTray")
            End
        End If

        '設定したインターバル(秒)をタイマーに設定する
        If Not OptionLogInterval > 0 Then
            NotifyIcon1.Visible = False

            MsgBox("ログインターバルの設定値が異常です。設定(LOGINT)を確認してください。",
                   vbOKOnly,
                   "PingTray")
            End
        Else
            Timer2.Interval = OptionLogInterval
        End If
    End Sub

    Private Function ExecDeleteLog() As Integer
        Dim SystemDate As DateTime = DateTime.Parse(Now.ToString("yyyy/MM/dd"))
        Dim RemoveDate As DateTime 'ログ削除起点
        Dim FolderList As String() 'フォルダリスト
        Dim FolderName As String 'フォルダ名称
        Dim i As Long 'ループカウンタ

        'INIファイルの設定から削除の起点を取得する
        RemoveDate = SystemDate.AddDays(OptionLogDay * -1)

        'ログフォルダからフォルダの一覧を配列に取得する(削除しないフォルダも含む)
        FolderList = Directory.GetDirectories(OptionLogDir, "*.*")

        If FolderList.Count <> 0 Then
            'フォルダの分だけ繰り返す
            For i = 0 To FolderList.Count - 1
                'フォルダの年月日8桁だけ取得する
                FolderName = FolderList.GetValue(i).ToString
                FolderName = Path.GetFileName(FolderName)

                'フォルダ名が「PingTray_」の場合
                If Strings.Left(FolderName, 9) = "PingTray_" Then

                    '数値だけだけの場合に実行
                    If IsNumeric(Strings.Right(FolderName, 8)) Then

                        'フォルダの名称(年月日8桁)が削除起点よりも古いのを数値型で判定してその場合削除を実行する
                        If Integer.Parse(CInt(RemoveDate.ToString("yyyyMMdd"))) > Integer.Parse(CInt(Strings.Right(FolderName, 8))) Then
                            Directory.Delete(FolderList.GetValue(i).ToString, True)
                            Debug.Print("Delete:" & FolderList.GetValue(i).ToString)
                        End If

                    End If

                End If

            Next

        End If

    End Function
End Class